# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kivy_garden', 'kivy_garden.i18n']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'kivy-garden.i18n',
    'version': '0.1.0.dev1',
    'description': 'i18n for Kivy',
    'long_description': '# i18n\n\nKivyアプリの多言語化は面倒くさいです。\nというのも只表示する文字列を切り替えればいいわけではないからです。\n**Kivyは文字列の描画に必要なフォントを自動で選んでくれはしないので、アプリ側がKivyに教えてあげなければいけません。**\nそれを怠れば画面には文字に代わって豆腐が表示されてしまいます。\n\nまた多くのKivyアプリはフォントをアプリに詰め込むという方法を採っていますが、これも多言語化の際には問題となります。\nフォントを幾つもアプリに詰め込めんでアプリのサイズが数十MB膨れ上がればアプリの利用者は喜ばないでしょう。\n\nそこでこのmoduleの出番です。\nこのmoduleは文字列を切り替える機能に加えて、**OSにinstall済のフォントの中から各言語用の物を自動で選んでくれます。**\nなのでアプリにフォントを詰め込む必要はもうありません。\n\n## install方法\n\n```\npip install --pre kivy_garden.i18n\n```\n\n## 使い方\n\n### フォントの検索\n\n表示する文字列の切り替えとフォントの切り替えは別々の機能であり、どちらか片方だけ使うこともできます。\nまた多言語化には興味はないけどOSにinstall済のフォントを利用して少しでもアプリの容量を減らしたいという人も居るでしょう。\nそういった人には以下のような使い方がおすすめです。\n\n\n```python\n# OSにinstall済の日本語フォントの内 最初に見つけた物を利用する例\n\nfrom kivy_garden.i18n.fontfinder import enum_fonts_from_lang\n\nfont = next(enum_fonts_from_lang(\'ja\'), None)\nif font is None:\n    print(f"日本語フォントが見つかりませんでした")\nelse:\n    label.font_name = font.name\n```\n\n### フォントの切り替え\n\nそして多言語化させたい場合は以下のようにします。\n\n```python\nfrom kivy_garden.i18n.localizer import KXLocalizer\n\nloc = KXLocalizer()\nloc.lang = \'ja\'\nprint(loc.font_name)  # => 何かの日本語フォント名が出力される\nloc.lang = \'zh\'\nprint(loc.font_name)  # => 何かの中文フォント名が出力される\n```\n\n重要なのが`KXLocalizer`が`EventDispatcher`な事で\n\n```python\nclass KXLocalizer(EventDispatcher):\n    lang = StringProperty(...)\n    font_name = StringProperty(...)\n    _ = ObjectProperty(...)\n```\n\nbindingを利かせれば`lang`を切り替えた時に`Label`の`font_name`も自動で切り替わるようにできます。\n\n```python\nfrom kivy.app import App\nfrom kivy.lang import Builder\nfrom kivy_garden.i18n.localizer import KXLocalizer\n\n\nKV_CODE = \'\'\'\nLabel:\n    font_name: app.loc.font_name\n\'\'\'\n\n\nclass SampleApp(App):\n    def build(self):\n        self.loc = KXLocalizer()\n        return Builder.load_string(KV_CODE)\n\n    def on_start(self):\n        self.loc.lang = \'ja\'  # bindingによりLabelには自動で日本語フォントが適用される\n```\n\nまた`KXLocalizer.install()`を用いる事で`#:import`無しで`KXLocalizer`をKv言語内で直接参照できるようにもできます。\nただしglobal変数を書き換える行為なので使用は自己責任で。\n\n```python\n# 上のcodeど同等のcode\nfrom kivy.app import App\nfrom kivy.lang import Builder\nfrom kivy_garden.i18n.localizer import KXLocalizer\n\n\nKV_CODE = \'\'\'\nLabel:\n    font_name: l.font_name  # import無しで参照 !!\n\'\'\'\n\nclass SampleApp(App):\n    def build(self):\n        self.loc = KXLocalizer()\n        self.loc.install(name=\'l\')  # install !!\n        return Builder.load_string(KV_CODE)\n\n    def on_start(self):\n        self.loc.lang = \'ja\'\n```\n\n### 翻訳(文字列の切り替え)\n\nそして肝心の翻訳ですが、これは`KXLocalizer`に翻訳者(Translator)を与える事で実現できます。\n現在ある翻訳者は\n\n- `gettext`を利用する`GettextTranslator`と\n- 辞書を用いた単純な仕組みの`DictBasedTranslator`\n\nの二種で、ここでは`DictBasedTranslator`を用いた方法を解説します。\n`GettextTranslator`に関しては[gettext_example](https://github.com/gottadiveintopython/i18n/blob/main/examples/gettext_example.py)を参照して下さい。\n\nまず必要となるのは以下のような辞書型の翻訳表です。\n\n```python\n翻訳表 = {\n    \'tiger\': {\n        \'zh\': \'老虎\',\n        \'ja\': \'虎\',\n        \'en\': \'tiger\',\n    },\n    \'apple\': {\n        \'zh\': \'蘋果\',\n        \'ja\': \'りんご\',\n        \'en\': \'apple\',\n    },\n}\n```\n\npythonの辞書literalは書きづらいので実際にはYAML形式で翻訳表を作り、それをpythonの辞書に変換した方が良いかもしれません。\n\n```yaml\ntiger:\n  zh: 老虎\n  ja: 虎\n  en: tiger\napple:\n  zh: 蘋果\n  ja: りんご\n  en: apple\n```\n\n```python\nimport yaml\n翻訳表 = yaml.safe_load(YAML文字列)\n```\n\n翻訳表ができたら後は以下のようにして`KXLocalizer`に与えるだけです。\n\n```python\nfrom kivy_garden.i18n.localizer import KXLocalizer, DictBasedTranslator\nloc = KXLocalizer(translator=DictBasedTranslator(翻訳表))\n```\n\nすると既に`loc.lang`(現在の言語)に基づいて適切な翻訳結果が得られるようになっています。\n\n```python\nloc.lang = \'ja\'\nprint(loc._("tiger"))  # => 虎\nprint(loc._("apple"))  # => りんご\nprint(loc.font_name)   # => 何かの日本語フォント名\nloc.lang = \'zh\'\nprint(loc._("tiger"))  # => 老虎\nprint(loc._("apple"))  # => 蘋果\nprint(loc.font_name)   # => 何かの中文フォント名\n```\n\nそしてこれもなのですがbindingを利かせてあげれば`loc.lang`(現在の言語)に連動して`Label`の`text`と`font_name`が自動で更新されます。\n\n```python\nfrom kivy.app import App\nfrom kivy.lang import Builder\nfrom kivy_garden.i18n.localizer import KXLocalizer, DictBasedTranslator\n\n\nKV_CODE = \'\'\'\nLabel:\n    font_name: l.font_name\n    text: l._("apple")\n\'\'\'\n\n\nclass SampleApp(App):\n    def build(self):\n        self.loc = KXLocalizer(translator=DictBasedTranslator(翻訳表))\n        self.loc.install(name=\'l\')\n        return Builder.load_string(KV_CODE)\n\n    def on_start(self):\n        # Labelのfont_nameに日本語フォントが、textには りんご が入る\n        self.loc.lang = \'ja\'\n\n        # Labelのfont_nameに英文フォントが、textには apple が入る\n        self.loc.lang = \'en\'\n```\n\nこのように`EventDispacther`のbindingの仕組みを利用すると**アプリの実行中に表示言語を自由に切り替えられるのがこのmoduleの強みとなります**。\nもちろん言語切替時にアプリの再起動を求めるつもりなのであれば無理にbindingを利かせる必要はありません。\n\n以上がこのmoduleの基本的な使い方となります。\n\n## 小道具\n\n`xgettext`は翻訳の必要性のある文字列をpythonソース中から抜き出してくれる素晴らしい道具ですが、\n文字列literalの中までは見てくれないという欠点があります。\nすなわち以下のcodeにおいて\n\n```python\nKV_CODE = \'\'\'\nBoxLayout:\n    Label:\n        text: _("ABC")\n    Label:\n        text: _("DEF")\n\'\'\'\n\n_("123")\n```\n\n`123`は抜き出してくれても`ABC`と`DEF`は抜き出してくれません。\nというわけでそういったことをしてくれる道具を作りました。\n\n```\npython -m kivy_gardem.i18n.extract_msgids_from_string_literals 上記のpythonファイル > ./output.py\n```\n\n```python\n# output.pyの中身\n_("ABC")\n_("DEF")\n```\n\nこのように文字列literal内の翻訳対象文字列をその外に抜き出してくれるので、\nそれを`xgettext`に喰わせてあげれば取りこぼしをせずに済みます。\n',
    'author': 'Nattōsai Mitō',
    'author_email': 'flow4re2c@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gottadiveintopython/i18n',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
